/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_pwm.c
*
* @brief This file provides PWM module functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "bldc_pwm.h"
#include "bldc_keys.h"
#include "bldc_mcu_config.h"
#include "bldc_parameters_define.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Enable or disable PWM channel output according to the input mark.
*
* @param[in] enable: Enable mark, 'enable' = DISABLE = 0, will mask the PWM channel output; 'enable' = ENABLE = 1, will make the PWM channel output.
* @return none
*/
void PWM_Output(uint8_t enable)
{
    if (enable)
    {
        PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL0);
        PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL1);
        PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL2);
        PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL3);
        PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL4);
        PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL5);
    }
    else
    {
        PWM_OutputMaskSet(PWM2, PWM_CHANNEL_CHANNEL0);
        PWM_OutputMaskSet(PWM2, PWM_CHANNEL_CHANNEL1);
        PWM_OutputMaskSet(PWM2, PWM_CHANNEL_CHANNEL2);
        PWM_OutputMaskSet(PWM2, PWM_CHANNEL_CHANNEL3);
        PWM_OutputMaskSet(PWM2, PWM_CHANNEL_CHANNEL4);
        PWM_OutputMaskSet(PWM2, PWM_CHANNEL_CHANNEL5);
    }
}

/*!
* @brief Mask or Clear Mask PWM channel output according to the bldc vector.
*
* @param[in] Vector: six step vector of Bldc in BLDC_COMMUTATION_VECTOR.
* @return none
*/
void PWM_MaskCtrl(uint8_t Vector)
{
    switch (Vector)
    {
    case AH_PWM_BL_ON: case BH_PWM_AL_ON:
    PWM_OutputMaskSet(PWM2, PWM_CHANNEL_CHANNEL4);
    PWM_OutputMaskSet(PWM2, PWM_CHANNEL_CHANNEL5);
    PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL0);
    PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL1);
    PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL2);
    PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL3);
    break;

    case AH_PWM_CL_ON: case CH_PWM_AL_ON:
    PWM_OutputMaskSet(PWM2, PWM_CHANNEL_CHANNEL2);
    PWM_OutputMaskSet(PWM2, PWM_CHANNEL_CHANNEL3);
    PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL0);
    PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL1);
    PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL4);
    PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL5);
    break;

    case BH_PWM_CL_ON: case CH_PWM_BL_ON:
    PWM_OutputMaskSet(PWM2, PWM_CHANNEL_CHANNEL0);
    PWM_OutputMaskSet(PWM2, PWM_CHANNEL_CHANNEL1);
    PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL2);
    PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL3);
    PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL4);
    PWM_ClrOutputMask(PWM2, PWM_CHANNEL_CHANNEL5);
    break;

    default:
    break;
    }
}

/*!
* @brief Clear pwm duty and stop bldc motor.
*
* @param[in] none
* @return none
*/
void BLDC_Stop(void)
{
//    PWM2->CHOSWCR = ALL_CH_OFF;
    PWM_Output(DISABLE);
    UpDatePwmDuty(0);

    PWM_SoftwareSync(PWM2);
}

/*!
* @brief Get PWM module effective duty in 10000 times
*
* @param[in] bldcCtrl: pointer to BLDC_VARS_CTRL structure
* @return pwm effective duty
*/
uint16_t Get_EffectiveDuty(BLDC_VARS_CTRL *bldcCtrl)
{
    uint32_t temp_PwmCh0Val;
    uint32_t temp_PwmCh1Val;
    uint32_t temp_PwmMcvr;
#if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)

#else
    temp_PwmCh0Val = PWM2->CHANNELS[0].CHV;
    temp_PwmCh1Val = PWM2->CHANNELS[1].CHV;
    temp_PwmMcvr = PWM2->MCVR;
    #if (PHASE_UVW_POLARITY == ACTIVE_LOW)
    bldcCtrl->pwmOutDuty = 10000 * (temp_PwmCh1Val - temp_PwmCh0Val) / temp_PwmMcvr;
    #else
    bldcCtrl->pwmOutDuty = 10000 * (temp_PwmMcvr + temp_PwmCh0Val - temp_PwmCh1Val) / temp_PwmMcvr;
    #endif
#endif

    /* enlarge 10000 times, 1234 means 12.34% */
    return bldcCtrl->pwmOutDuty;
}

/*!
* @brief Update PWM output duty by manual settig or current PID calculate.
*
* @param[in] pwmDuty: range: 0 ~ PWMx->MCVR, corresponding to 0 ~ 100% duty cycle
* @return none
*/
void UpDatePwmDuty(uint16_t pwmDuty)
{
    if (pwmDuty > (PWM2->MCVR - 1))
    {
        pwmDuty = (PWM2->MCVR - 1);
    }
    else if (pwmDuty < 1)
    {
        pwmDuty = 1;
    }
    else
    {

    }
#if (defined BLDC_AFTER_FLOW_CONTROL)
    if (g_bemfControl.commutateState == 1)
    {
        if (g_bemfControl.changePhaseTime > ((g_bemfControl.afterFlowTime * AFTER_FLOW_ANGLE_NUMERATOR) >> AFTER_FLOW_ANGLE_PSRC))
        {
            switch (g_bemfControl.currentDriverVector)
            {
            case AH_PWM_BL_ON:
            PWM2->CHOSWCR = CH0_MODULATION_CH3_ON;
            break;

            case AH_PWM_CL_ON:
            PWM2->CHOSWCR = CH0_MODULATION_CH5_ON;
            break;

            case BH_PWM_CL_ON:
            PWM2->CHOSWCR = CH2_MODULATION_CH5_ON;
            break;

            case BH_PWM_AL_ON:
            PWM2->CHOSWCR = CH2_MODULATION_CH1_ON;
            break;

            case CH_PWM_AL_ON:
            PWM2->CHOSWCR = CH4_MODULATION_CH1_ON;
            break;

            case CH_PWM_BL_ON:
            PWM2->CHOSWCR = CH4_MODULATION_CH3_ON;
            break;

            default:
            break;
            }
            g_bemfControl.commutateState = 2;
        }
    }
#endif

#if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)

#else
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL1, (PWM_PERIOD_VALUE + PWM_PERIOD_VALUE - pwmDuty) >> 1);
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL3, (PWM_PERIOD_VALUE + PWM_PERIOD_VALUE - pwmDuty) >> 1);
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL5, (PWM_PERIOD_VALUE + PWM_PERIOD_VALUE - pwmDuty) >> 1);
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL0, (pwmDuty >> 1));
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL2, (pwmDuty >> 1));
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL4, (pwmDuty >> 1));
    #else
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL1, (PWM_PERIOD_VALUE + pwmDuty) >> 1);
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL3, (PWM_PERIOD_VALUE + pwmDuty) >> 1);
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL5, (PWM_PERIOD_VALUE + pwmDuty) >> 1);
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL0, (PWM_PERIOD_VALUE - pwmDuty) >> 1);
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL2, (PWM_PERIOD_VALUE - pwmDuty) >> 1);
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL4, (PWM_PERIOD_VALUE - pwmDuty) >> 1);
    #endif
#endif

    PWM_SoftwareSync(PWM2);
}

/*!
* @brief Control afterflow time in bldc sensorless control, reduce afterflow time.
*
* @param[in] Vector: current magnetic vector in bldc sensorless control
* @param[in] motorDir: motor run direction
* @return none
*/
void AfterFlow_Control(uint8_t Vector, int16_t motorDir)
{
    if (motorDir == 1)
    {
        switch (Vector)
        {
        case AH_PWM_BL_ON:
        #if (defined FALL_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWM2->CHOSWCR = PWM_CH0_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWM2->CHOSWCR = CH0_MODULATION_CH3_ON;
        #endif
        PWM_SoftwareSync(PWM2);
        break;

        case AH_PWM_CL_ON:
        #if (defined RISE_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWM2->CHOSWCR = PWM_CH5_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWM2->CHOSWCR = CH0_MODULATION_CH5_ON;
        #endif
        PWM_SoftwareSync(PWM2);
        break;

        case BH_PWM_CL_ON:
        #if (defined FALL_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWM2->CHOSWCR = PWM_CH2_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWM2->CHOSWCR = CH2_MODULATION_CH5_ON;
        #endif
        PWM_SoftwareSync(PWM2);
        break;

        case BH_PWM_AL_ON:
        #if (defined RISE_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWM2->CHOSWCR = PWM_CH1_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWM2->CHOSWCR = CH2_MODULATION_CH1_ON;
        #endif
        PWM_SoftwareSync(PWM2);
        break;

        case CH_PWM_AL_ON:
        #if (defined FALL_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWM2->CHOSWCR = PWM_CH4_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWM2->CHOSWCR = CH4_MODULATION_CH1_ON;
        #endif
        PWM_SoftwareSync(PWM2);
        break;

        case CH_PWM_BL_ON:
        #if (defined RISE_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWM2->CHOSWCR = PWM_CH3_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWM2->CHOSWCR = CH4_MODULATION_CH3_ON;
        #endif
        PWM_SoftwareSync(PWM2);
        break;

        default:
        break;
        }
    }
    else
    {
        switch (Vector)
        {
        case AH_PWM_BL_ON:
        #if (defined RISE_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWM2->CHOSWCR = PWM_CH3_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWM2->CHOSWCR = CH0_MODULATION_CH3_ON;
        #endif
        PWM_SoftwareSync(PWM2);
        break;

        case AH_PWM_CL_ON:
        #if (defined FALL_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWM2->CHOSWCR = PWM_CH0_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWM2->CHOSWCR = CH0_MODULATION_CH5_ON;
        #endif
        PWM_SoftwareSync(PWM2);
        break;

        case BH_PWM_CL_ON:
        #if (defined RISE_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWM2->CHOSWCR = PWM_CH5_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWM2->CHOSWCR = CH2_MODULATION_CH5_ON;
        #endif
        PWM_SoftwareSync(PWM2);
        break;

        case BH_PWM_AL_ON:
        #if (defined FALL_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWM2->CHOSWCR = PWM_CH2_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWM2->CHOSWCR = CH2_MODULATION_CH1_ON;
        #endif
        PWM_SoftwareSync(PWM2);
        break;

        case CH_PWM_AL_ON:
        #if (defined RISE_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWM2->CHOSWCR = PWM_CH1_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWM2->CHOSWCR = CH4_MODULATION_CH1_ON;
        #endif
        PWM_SoftwareSync(PWM2);
        break;

        case CH_PWM_BL_ON:
        #if (defined FALL_AFTER_FLOW)
        if (Get_McStatus() == RUN)
        {
            PWM2->CHOSWCR = PWM_CH4_ON;
            g_bemfControl.commutateState = 1;
        }
        #else
        PWM2->CHOSWCR = CH4_MODULATION_CH3_ON;
        #endif
        PWM_SoftwareSync(PWM2);
        break;

        default:
        break;
        }
    }
}

/*!
* @brief update PWM channel output by six step square wave magnetic vector to driver MOS.
*
* @param[in] ChangePhaseVector: current magnetic vector in bldc control
* @return none
*/
void MosDriver(uint8_t ChangePhaseVector)
{
#if (defined BLDC_AFTER_FLOW_CONTROL)
    AfterFlow_Control(ChangePhaseVector, pBldcVarsCfg->motorDir);
#else

    PWM_MaskCtrl(ChangePhaseVector);

    switch (ChangePhaseVector)
    {
    case AH_PWM_BL_ON:
    PWM2->CHOSWCR = CH0_MODULATION_CH3_ON;
    PWM_SoftwareSync(PWM2);
    break;

    case AH_PWM_CL_ON:
    PWM2->CHOSWCR = CH0_MODULATION_CH5_ON;
    PWM_SoftwareSync(PWM2);
    break;

    case BH_PWM_CL_ON:
    PWM2->CHOSWCR = CH2_MODULATION_CH5_ON;
    PWM_SoftwareSync(PWM2);
    break;

    case BH_PWM_AL_ON:
    PWM2->CHOSWCR = CH2_MODULATION_CH1_ON;
    PWM_SoftwareSync(PWM2);
    break;

    case CH_PWM_AL_ON:
    PWM2->CHOSWCR = CH4_MODULATION_CH1_ON;
    PWM_SoftwareSync(PWM2);
    break;

    case CH_PWM_BL_ON:
    PWM2->CHOSWCR = CH4_MODULATION_CH3_ON;
    PWM_SoftwareSync(PWM2);
    break;

    default:
    break;
    }
#endif
}

/*!
* @brief update six step square wave magnetic vector by current hall status.
*
* @param[in] motorDir: motor run direction
* @param[in] hallStatus: current rotor position hall status
* @return none
*/
void BLDC_Commutate_BaseSector(int16_t motorDir, uint8_t hallStatus)
{
    if (motorDir != -1)
    {
        switch (hallStatus)
        {
        case 1:
            /* B+,C- */
            g_hallControl.currentDriverVector = (uint8_t)BH_PWM_CL_ON;
            break;

        case 2:
            /* A+,B- */
            g_hallControl.currentDriverVector = (uint8_t)AH_PWM_BL_ON;
            break;

        case 3:
            /* A+,C- */
            g_hallControl.currentDriverVector = (uint8_t)AH_PWM_CL_ON;
            break;

        case 4:
            /* C+,A- */
            g_hallControl.currentDriverVector = (uint8_t)CH_PWM_AL_ON;
            break;

        case 5:
            /* B+,A- */
            g_hallControl.currentDriverVector = (uint8_t)BH_PWM_AL_ON;
            break;

        case 6:
            /* C+,B- */
            g_hallControl.currentDriverVector = (uint8_t)CH_PWM_BL_ON;
            break;

        default:
            BLDC_Stop();
            break;
        }
    }
    else
    {
        switch (hallStatus)
        {
        case 1:
            /* C+,B- */
            g_hallControl.currentDriverVector = (uint8_t)CH_PWM_BL_ON;
            break;

        case 2:
            /* B+,A- */
            g_hallControl.currentDriverVector = (uint8_t)BH_PWM_AL_ON;
            break;

        case 3:
            /* C+,A- */
            g_hallControl.currentDriverVector = (uint8_t)CH_PWM_AL_ON;
            break;

        case 4:
            /* A+,C- */
            g_hallControl.currentDriverVector = (uint8_t)AH_PWM_CL_ON;
            break;

        case 5:
            /* A+,B- */
            g_hallControl.currentDriverVector = (uint8_t)AH_PWM_BL_ON;
            break;

        case 6:
            /* B+,C- */
            g_hallControl.currentDriverVector = (uint8_t)BH_PWM_CL_ON;
            break;

        default:
            BLDC_Stop();
            break;
        }
    }
    MosDriver(g_hallControl.currentDriverVector);
}

/* =============================================  EOF  ============================================== */
