/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

#ifndef __FOC_MCU_CONFIG_H__
#define __FOC_MCU_CONFIG_H__
/*!
* @file foc_mcu_config.h
*
* @brief This file provides MCU peripheral initialization functions interfaces.
*
*/

/* ===========================================  Includes  =========================================== */
#include "ac78xx_pwm.h"
#include "cur_restruct.h"
#include "drive_parameters_define.h"
#include "adcdrv.h"

/* ============================================  Define  ============================================ */
/*!
* NOTICE!
* THE PARAMETERS IN THIS FILE CAN ONLY BE CHANGED ACCORDING TO THE SUPPORT OF TECHNICAL PERSONNEL OR REFERENCE MANUAL!
* DO NOT CHANGE AT WILL, OTHERWISE THE MCU PERIPHERAL MAY NOT WORK PROPERLY!
*/

#define PWM_OUTPUT_DISABLE    0
#define PWM_OUTPUT_ENABLE     1

/* define ADC Conversion channel */
#define CURRENT_IA              ADC_CHANNEL_AD5
#define CURRENT_IB              ADC_CHANNEL_AD4
#define CURRENT_IC              ADC_CHANNEL_AD6
#define VOLTAGE_VBUS            ADC_CHANNEL_AD8
#define CURRENT_IBUS            ADC_CHANNEL_AD7
#define BEMF_UA                 ADC_CHANNEL_AD0
#define BEMF_UB                 ADC_CHANNEL_AD1
#define BEMF_UC                 ADC_CHANNEL_AD2
#define BEMF_UREF               ADC_CHANNEL_AD3

/* define SVPWM Debug GPIO */
#define DEBUG_GPIO              GPIO_PA5

/* define SPI Interface Gpio */
#define SPI_NSS                 GPIO_PA0
#define SPI_SCK                 GPIO_PA1
#define SPI_MISO                GPIO_PA2
#define SPI_MOSI                GPIO_PA3

/* define phase current detection ADC */
/* ADC_IN4 */
#define IA_ADC                  GPIO_PA12
/* ADC_IN5 */
#define IB_ADC                  GPIO_PA11
/* ADC_IN6 */
#define IC_ADC                  GPIO_PA13

/* define back EMF detection ADC */
#define BEMF_U                  GPIO_PA7
#define BEMF_V                  GPIO_PA8
#define BEMF_W                  GPIO_PA9
#define BEMF_REF                GPIO_PA10

/* define bus voltage detection, ADC_IN8 */
#define VBUS_ADC                GPIO_PA15

/* define bus current detection, ADC_IN7 */
#define IAVE_ADC                GPIO_PA14

/* temperature sample, ADC_IN11 */
#define TEMP_ADC                GPIO_PB1

/* VR1 Knob sample, ADC_IN9 */
#define VR_ADC                  GPIO_PB0

/* define Hall Input A/B/C */
#define SA_Hall                 GPIO_PB8
#define SB_Hall                 GPIO_PB3
#define SC_Hall                 GPIO_PB4

//define ENCODER Input A/B/Z
/* PWM1_CH0 */
#define A_ENCODER               GPIO_PB9
/* PWM1_CH1 */
#define B_ENCODER               GPIO_PB10
/* ADC_IN10 */
#define Z_ENCODER               GPIO_PB2
/* define Debug Gpio */
/* PWM0_CH0 */
#define DEBUG_PWM1              GPIO_PB9
/* PWM0_CH1 */
#define DEBUG_PWM2              GPIO_PB10

/* define PWM2_CH0~CH5 Output */
#define AH_PWM                  GPIO_PB15
#define AL_PWM                  GPIO_PC0
#define BH_PWM                  GPIO_PB11
#define BL_PWM                  GPIO_PB12
#define CH_PWM                  GPIO_PB13
#define CL_PWM                  GPIO_PB14

/* define motor overload */
#define TRAP_PWM_FAULT          GPIO_PD0

/* define UART */
#define UART2_TX                GPIO_PD1
#define UART2_RX                GPIO_PD2

#define UART1_TX                GPIO_PB5
#define UART1_RX                GPIO_PB6
#define UART1_RTS               GPIO_PB7

#define UART4_TX                GPIO_PC15
#define UART4_RX                GPIO_PC14
#define UART4_RTS               GPIO_PC10

/* define motor mode select gpio */
/* UART4_TX */
#define MOTOR_MODE_SELECT_0     GPIO_PC4
/* UART4_RX */
#define MOTOR_MODE_SELECT_1     GPIO_PC5
/* I2C2_SDA */
#define MOTOR_MODE_SELECT_2     GPIO_PC6

/* LED */
#define LATCH_STCP              GPIO_PA6
#define LATCH_SHCP              GPIO_PA8
#define LED_DI                  GPIO_PA7
#define KEY_DO                  GPIO_PA9

#define MC_IO_1             GPIO_SetPinBits(GPIO_PC2)        /*!< Set GPIO PB-2 bit */
#define MC_IO_0             GPIO_ResetPinBits(GPIO_PC2)      /*!< Reset GPIO PB-2 bit */

/* define LED2 gpio PC4 */
//#define LED2                    GPIO_PC4

/* define PWM Modular parameters */
#define MOTOR_BUS_CLK           APB_BUS_FREQ
#define PWM_FREQ                PWM_FREQ_SETTING                                                /*!< in Hz, 16k/8kHz */

#if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
#define PWM_PERIOD_VALUE        (((MOTOR_BUS_CLK / (PWM_FREQ * (PWM_PRSC + 1))) >> 1) - 1)      /*!< calculate pwm period value */
#else
#define PWM_PERIOD_VALUE        ((MOTOR_BUS_CLK / (PWM_FREQ * (PWM_PRSC + 1))) - 1)
#endif

/* Set PWM dead time */
/*!<
* PWM_DEAD_TIME count value limitted from 0 to 63(required by register)
* PWM_DEAD_TIME count prescalerValue limitted from 0 to 3(required by register),default PWM_DEADTIME_DIVID_4(prescalerValue = 2).
* SW_DEAD_TIME_NS = 1000  //1000=1us 1500=1.5us
* PWM_DEAD_TIME = ((SW_DEAD_TIME_NS / 1000) * (APB_BUS_FREQ / PWM_DEADTIME_DIVID_4 / 1000000UL))
* PWM_DEADTIME_DIVID_1 = 1
* PWM_DEADTIME_DIVID_4 = 4
* PWM_DEADTIME_DIVID_16 = 16
*/
#define PWM_DEAD_TIME            (12 * SW_DEAD_TIME_NS / 1000)                                  /*!< default dead time set to 1us */

#define TIMER_1MS                (APB_BUS_FREQ / 1000 - 1)                                      /*!< Timer count value of 1 ms */
#define TIMER_2MS                (APB_BUS_FREQ / 500 - 1)                                       /*!< Timer count value of 2 ms */
#define TIMER_XUS                (12)                                                           /*!< User defined time unit us */
#define TIMER_12US               (APB_BUS_FREQ / (1000000 / TIMER_XUS) - 1)                     /*!< Timer count value of user defined time unit us */
#define TIMER_T_METHOD_US        (APB_BUS_FREQ / T_METHOD_HZ - 1)                               /*!< Timer count value of M_T speed measure time unit us */
#define M_T_PERIOD               TIMER_T_METHOD_US                                              /*!< Period of Timer Use to measure time in M_T speed measure */

/* define ADC Modular parameters */
/* define ADC Calbiration Conversion times */
#define NUM_OF_CONVERSIONS      32                                                              /*!< ADC Calbiration Conversion times */

/* signle shunt sample current restruct */
#define DEADTIME                ((SW_DEAD_TIME_NS / 1000.0) * (MOTOR_BUS_CLK /1000000))         /*!< Dead time for 1 shunt sample */
#define RISETIME                ((SAMPLE_RISE_TIME / 1000.0) * (MOTOR_BUS_CLK /1000000))        /*!< Rising time for 1 shunt sample */
#define AD_SAMPLE_TIME          ((SAMPLE_HOLD_TIME / 1000.0) * (MOTOR_BUS_CLK /1000000))        /*!< Sampling time for 1 shunt sample */
#define SAMPLE_WINDOW_US        (DEADTIME + RISETIME + AD_SAMPLE_TIME)                          /*!< Sampling window time for 1 shunt sample, 2.5us */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
/*!
* @brief ADC Regular sampling Buffer.
*/
extern uint32_t g_ADCRegularBuffer[5];

/* ====================================  Functions declaration  ===================================== */
/*!
* @brief FOC PWM Module Initialize.
*
* @param[in] none
* @return none
*/
void FOC_PWM_Initialize(void);

/*!
* @brief CTU Module Initialize
*
* @param[in] none
* @return none
*/
void FOC_CTU_Initialize(void);

/*!
* @brief Calbiration ADC to get adc channel offset value
*
* @param[in] sequencechannel : ADC Inject Group sequence num, 0-3
* @return dcoffset: ADC Inject Group channel offset
*/
int16_t MC_ADC_Calbiration(uint8_t sequencechannel);

/*!
* @brief ADC Initialize, first calibration current Offset, then config adc as external trigger mode.
*
* @param[in] none
* @return none
*/
void FOC_ADC_Initialize(void);

/*!
* @brief Peripheral Initialize in foc App.
*
* @param[in] none
* @return none
*/
void FOC_Peripheral_Initialize(void);

/*!
* @brief Get phase current sampling offset value.
*
* @param[in] adcSampleReal: pointer to ADC_SAMPLE structure
* @return none
*/
void FOC_ADC_Calbiration(ADC_SAMPLE *adcSampleReal);

/*!
* @brief Power board relay action control.
*
* @param[in] none
* @return none
*/
void Board_Relay_GPIO_Action(void);

/* ======================================  Functions define  ======================================== */

#endif /* __FOC_MCU_CONFIG_H__ */

/* =============================================  EOF  ============================================== */
