/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file foc_keys.c
*
* @brief This file provides keys control functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "ac78xx_gpio.h"
#include "foc_keys.h"
#include "foc_app.h"
#include "foc_ui_uart.h"
#include "param_identify_api.h"

/* ============================================  Define  ============================================ */
#define KEY_DEBOUNCE                5                                   /*!< Keys action debounce delay */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
KEY_STATUS g_keyStatus;                                                 /*!< Panel keys status structure */
MC_STATUS g_mcStatus = POWER_ON;                                        /*!< define motor operation status */
MC_STATUS g_mcBufStatus = TEMPORARY_BUFFER;                             /*!< define motor operation buffer status */

uint16_t g_dataMask[6] = {0xdf00, 0xef00, 0xf700, 0xfb00, 0Xfd00};      /*!< Panel data */

uint16_t DSP2[6] = {LEDBL, LED7, LED8, LED0, LED1};                     /*!< Panel display */
uint16_t g_keyInput = 0x0000;                                           /*!< Panel keys input */
uint16_t g_keyBuffer[KEY_DEBOUNCE] = {0x0000};                          /*!< Panel keys status buffer */
uint16_t g_keyScanCnt = 0;                                              /*!< Panel keys action counter */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Control panel key GPIO configuration.
*
* @param[in] none
* @return none
*/
void Panel_Config(void)
{
    GPIO_SetDir(LATCH_STCP, 1);            /*!< set LED&KEY CTCP pins as output */
    GPIO_SetDir(LED_DI, 1);                /*!< set LED_DATA pins as output */
    GPIO_SetDir(LATCH_SHCP, 1);            /*!< set LED&KEY SHCP pins as output */
    GPIO_SetDir(KEY_DO, 0);                /*!< set KEY_DO pins as pull-up input */
    GPIO_SetPullupEnable(KEY_DO, ENABLE);  /*!< enable pull-up */
}

/*!
* @brief Get control panel key status.
*
* @param[in] none
* @return 0: function executed successfully
*/
uint8_t Panel_ReflashKeyStatus(void)
{
    uint16_t i;

    for (i = 0; i < (KEY_DEBOUNCE - 1); i++)
    {
        if (g_keyBuffer[i] != g_keyBuffer[i+1])
        {
            return 1;
        }
    }
    g_keyInput = g_keyBuffer[0];
    if (g_keyInput & MEMU_KEY)
    {
        g_keyStatus.memuKey = 1;
    }
    else
    {
        g_keyStatus.memuKey = 0;
    }

    if (g_keyInput & PLUS_KEY)
    {
        g_keyStatus.plusKey = 1;
    }
    else
    {
        g_keyStatus.plusKey = 0;
    }
    if (g_keyInput & SHIFT_KEY)
    {
        g_keyStatus.shiftKey = 1;
    }
    else
    {
        g_keyStatus.shiftKey = 0;
    }
    if (g_keyInput & STARTSTOP_KEY)
    {
        g_keyStatus.startStopKey = 1;
    }
    else
    {
        g_keyStatus.startStopKey = 0;
    }
    if (g_keyInput & MINUS_KEY)
    {
        g_keyStatus.minusKey = 1;
    }
    else
    {
        g_keyStatus.minusKey = 0;
    }
    if (g_keyInput & CONFIRM_KEY)
    {
        g_keyStatus.confirmKey = 1;
    }
    else
    {
        g_keyStatus.confirmKey = 0;
    }
    return 0;
}

/*!
* @brief Control panel refresh. Must refresh periodically.
*
* @param[in] none
* @return none
*/
void Panel_Refresh(void)
{
    static uint16_t s_DspNumber = 0;    /*!< Panel digital number */
    uint8_t i;
    uint8_t delay;
    uint16_t dsp, dsp_in=0;
    LED_165_SHL_IO_0;
    for (delay = 0; delay < 15; delay++);

    LED_CLK_IO_1 ;
    LED_165_SHL_IO_1;

    dsp_in = 0;
    s_DspNumber++;
    if (s_DspNumber >= 5)
    {
        s_DspNumber=0;
    }
    dsp = DSP2[s_DspNumber] | g_dataMask[s_DspNumber];
    i = 16;
    for (delay = 0; delay < 15; delay++);
    do
    {

        dsp_in = dsp_in << 1;
        if ( LED_SIN_165_IO == 0)
        {
            dsp_in = dsp_in + 1;
        }

        if ((dsp & 0x8000) == 0)
        {
            LED_SOUT_595_IO_1;
        }
        else
        {
            LED_SOUT_595_IO_0;
        }

        dsp = dsp << 1;
        for (delay = 0; delay < 8; delay++);

        LED_CLK_IO_0 ;
        for (delay = 0; delay < 15; delay++);
        LED_CLK_IO_1 ;
        for (delay = 0; delay < 7; delay++);
        i--;
    } while (i!=0);

    g_keyBuffer[g_keyScanCnt] = dsp_in;
    g_keyScanCnt ++;
    if (g_keyScanCnt >= KEY_DEBOUNCE)
    {
        g_keyScanCnt = 0;
    }
    Panel_ReflashKeyStatus();
    LED_165_SHL_IO_0;
    for (delay = 0; delay < 15; delay++);
    LED_165_SHL_IO_1;
}

/*!
* @brief Get the response of pressing each key on the control board.
*
* @param[in] none
* @return none
*/
void MC_Keys_Read(void)
{
    if ((g_keyStatus.startStopKey == 1) && (!s_keyStartPressed))
    {
        s_keyStartPressed = 1;
    }
    else if ((g_keyStatus.startStopKey == 0) && s_keyStartPressed)
    {
        s_keyStartPressed = 0;
        if (Get_McStatus() == IDLE)
        {
            if (g_paramIdentifyCtrl.identifyCtrl == 1)
            {
                PushIn_McBufStatus(IDENTIFY);
            }
            else if (defHallSensor.hallSelfLearnEnable == ENABLE)
            {
                PushIn_McBufStatus(HALL_SELFLEARN);
            }
            else
            {
                if (SHORT_BRAKE_CTRL & 0x01)
                {
                    #if (defined TAIL_HEAD_WIND)
                    PushIn_McBufStatus(SPEED_JUDGE);
                    #else
                    PushIn_McBufStatus(START_BRAKE);
                    #endif
                }
                else
                {
                    PushIn_McBufStatus(START);
                }

            }
        }
        else if ((Get_McStatus() == START_BRAKE) || (Get_McStatus() == STOP_BRAKE))
        {
            PushIn_McBufStatus(STOP);
        }
        else if (Get_McStatus() == RUN)
        {
            if (STOP_MODE == SLOW_DOWN_STOP)
            {
                PushIn_McBufStatus(SLOW_DOWN);
            }
            else if (SHORT_BRAKE_CTRL > 1)
            {
                PushIn_McBufStatus(STOP_BRAKE);
            }
            else
            {
                PushIn_McBufStatus(STOP);
            }
        }
        else if (Get_McStatus() == HALL_SELFLEARN)
        {
            defHallSensor.hallSelfLearnEnable = DISABLE;
            PushIn_McBufStatus(STOP);
        }
        else if (Get_McStatus() == IDENTIFY)
        {
            g_paramIdentifyCtrl.initCtrl = 1;
            PushIn_McBufStatus(STOP);
        }
        else
        {
            PushIn_McBufStatus(STOP);
        }
    }
    else if ((g_keyStatus.shiftKey == 1) && (!s_keyDirPressed))
    {
        s_keyDirPressed = 1;
    }
    else if ((g_keyStatus.shiftKey == 0)  && (s_keyDirPressed))
    {
        s_keyDirPressed = 0;
        if (pFocVarsCfg->motorDir == 1)
        {
            pFocVarsCfg->motorDir = -1;
        }
        else
        {
            pFocVarsCfg->motorDir = 1;
        }
//            g_speedCommand.speedTarget = Math_Qabs(g_speedCommand.speedTarget) * pFocVarsCfg->motorDir;
    }
    else if ((g_keyStatus.plusKey == 1) && (!s_keyUpPressed))
    {
        s_keyUpPressed = 1;
    }
    else if ((g_keyStatus.plusKey == 0) && (s_keyUpPressed))
    {
        s_keyUpPressed = 0;
        g_speedCommand.speedTarget += (pFocVarsCfg->motorDir * SPEED_CHANGE_STEP);
    }
    else if ((g_keyStatus.minusKey == 1) && (!s_keyDownPressed))
    {
        s_keyDownPressed = 1;
    }
    else if ((g_keyStatus.minusKey == 0) && (s_keyDownPressed))
    {
        s_keyDownPressed = 0;
        g_speedCommand.speedTarget -= (pFocVarsCfg->motorDir * SPEED_CHANGE_STEP);
    }
    else
    {
        /* do nothing */
    }

}

/*!
* @brief Get the main status of motor control right now.
*
* @param[in] none
* @return MC_STATUS: current motor control main status
*/
MC_STATUS Get_McStatus(void)
{
    return g_mcStatus;
}

/*!
* @brief Set the main status of motor control right now.
*
* @param[in] MC_STATUS: status to beset
* @return none
*/
void Set_McStatus(MC_STATUS status)
{
    g_mcStatus = status;
}

/*!
* @brief Push the buffer status in motor control main status buffer.
*
* @param[in] MC_STATUS: statusBuffer to store
* @return none
*/
void PushIn_McBufStatus(MC_STATUS statusBuffer)
{
    if (g_mcBufStatus == TEMPORARY_BUFFER)
    {
        g_mcBufStatus = statusBuffer;
    }
}

/*!
* @brief Pull out buffer status to refresh motor control main status.
*
* @param[in] none
* @return none
*/
void PullOut_McBufStatus(void)
{
    if (g_mcBufStatus != TEMPORARY_BUFFER)
    {
        g_mcStatus = g_mcBufStatus;
        g_mcBufStatus = TEMPORARY_BUFFER;
    }
}

/* =============================================  EOF  ============================================== */
