/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file foc_mcu_config.c
*
* @brief This file provides MCU peripheral initialization functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "ac78xx_gpio.h"
#include "ac78xx_timer.h"
#include "ac78xx_uart.h"
#include "ac78xx_pwdt.h"
#include "ac78xx_pwm.h"
#include "ac78xx_adc.h"
#include "ac78xx_ctu.h"
#include "ac78xx_acmp.h"
#include "ac78xx_dma.h"
#include "ac78xx_spi.h"
#include "globalvar.h"
#include "foc_isr.h"
#include "foc_app.h"
#include "foc_mcu_config.h"
#include "foc_keys.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
uint32_t g_ADCRegularBuffer[5] = {0};  /*!< Buffer to storage Regular Group ADC Value */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Power board relay action control.
*
* @param[in] none
* @return none
*/
void Board_Relay_GPIO_Action(void)
{
    /* Set SHORT pins as output */
    GPIO_SetFunc(GPIO_PC2, 0);
    GPIO_SetDir(GPIO_PC2, 1);
    /* Charging resistance relay disconnected! */
    MC_IO_0;
    /* Delay 2000ms to charge */
    mdelay(2000);
    /* Charging resistance relay short circuit! */
    MC_IO_1;
}

/*!
* @brief FOC Control Module GPIO Initialize.
*
* @param[in] none
* @return none
*/
void FOC_GPIO_Initialize(void)
{
    /* Setting control board panel GPIO pin */
    Panel_Config();
#ifdef USE_GPIO_CALCULATE_FOC_RUNTIME
    /* Setting the multiplexing function of GPIO pin */
    GPIO_SetFunc(DEBUG_GPIO, 0);
    /* Setting output as the attribute of GPIO pin */
    GPIO_SetDir(DEBUG_GPIO, 1);
#endif

    /* Setting the LED_2 to display operation states */
//    GPIO_SetFunc(LED_2, 0);
//    GPIO_SetDir(LED_2, 1);
}

/*!
* @brief FOC Control Module ACMP Initialize.
*
* @param[in] none
* @return none
*/
void FOC_ACMP_Initialize(void)
{
    ACMP_InitType acmpConfig = {0};
    ACMP_PollingType acmpPollConfig = {0};

    /* Init emf GPIO as ADC input */
    GPIO_SetFunc(BEMF_U, 1);
    GPIO_SetFunc(BEMF_V, 1);
    GPIO_SetFunc(BEMF_W, 1);
    GPIO_SetFunc(BEMF_REF, 1);
    ACMP_ModuleEnable();
    ACMP_ModeSel(ACMP_CHANNEL0, ACMP_NORMAL_MODE);
    /* ACMP Polling Mode Configuration */
    acmpPollConfig.posPollingEnable = ACMP_ENABLE;
    acmpPollConfig.negPollingEnable = ACMP_DISABLE;
    acmpPollConfig.pollingChannelSeq = 7;
    acmpPollConfig.pollingDiv = ACMP0_POLLING_FREQ_DIVIDER_256;
    acmpPollConfig.hallOutEn = ACMP_ENABLE;
    acmpPollConfig.hallAOutput = ACMP_INPUT_EXT0;
    acmpPollConfig.hallBOutput = ACMP_INPUT_EXT1;
    acmpPollConfig.hallCOutput = ACMP_INPUT_EXT2;
    ACMP_PollingModeInit(ACMP_CHANNEL0, &acmpPollConfig);
    /* ACMP Init Configuration */
    acmpConfig.negPin = ACMP_INPUT_EXT3;
    acmpConfig.edgeType = ACMP_INTERRUPT_EDGE_FALLING_RISING;
    acmpConfig.intEn = ACMP_DISABLE;
    acmpConfig.moduleEnable = ACMP_ENABLE;
    acmpConfig.pollEn = ACMP_ENABLE;
    ACMP_Init(ACMP_CHANNEL0, &acmpConfig);
}

/*!
* @brief FOC Control Module PWDT Initialize.
*
* @param[in] none
* @return none
*/
void FOC_PWDT_Initialize(void)
{
    PWDT_ConfigType  pwdtConfig;

    /*
    * GPIO Configuration for PWDT1
    * PC8     --> PWDT1_IN0
    * PC7     --> PWDT1_IN1
    * PC6     --> PWDT1_IN2
    */
    GPIO_SetFunc(SA_Hall, 1);
    GPIO_SetFunc(SB_Hall, 1);
    GPIO_SetFunc(SC_Hall, 1);

    /* Config HALL Interface */
    pwdtConfig.overflowIntEn = 1;
    /* enable data reday interrupt */
    pwdtConfig.readyIntEn = 1;
    /* enable PWDT module interrupt */
    pwdtConfig.pwdtIntEn = 1;
    /* enable PWDT module */
    pwdtConfig.pwdtEn = 1;
    /* start to measure pulse from first rising-edge and all dege */
    pwdtConfig.edgeMode = 1;
    pwdtConfig.clockPreScaler = PWDT_CLK_PRESCALER;
    /* PWDT clock source is bus clock */
    pwdtConfig.clockSource = PWDT_PCLKS_BUS;
    pwdtConfig.pwdtTimerEn = 0;
    /* Enable PWDT Hall Interface */
    pwdtConfig.pwdtHallEn = 1;
    /* Enable PWDT Hall Filter value = 10KHz */
    pwdtConfig.pwdtFilterEn = 1;
    pwdtConfig.pwdtFilterPsc = 5;
    pwdtConfig.pwdtFilterValue = 15;
    PWDT_SetCallback(FOC_PWDT_Callback);
    PWDT_Init(&pwdtConfig);
    NVIC_EnableIRQ(PWDT_IRQn);
    NVIC_SetPriority(PWDT_IRQn, 1);
}

/*!
* @brief FOC Control Encoder Mode Module PWM Initialize.
*
* @param[in] none
* @return none
*/
void FOC_Encoder_Initialize(void)
{
    /* Encoder config struct */
    QEI_ConfigType encoderConfig;

    /*
    * GPIO Configuration for ENCODER
    * PA1     --> PWM0_CH0
    * PA0     --> PWM0_CH1
    * PB13    --> ENCODER_Z
    */
    GPIO_SetFunc(A_ENCODER, 1);
    GPIO_SetFunc(B_ENCODER, 1);
    /* Use gpio interrrupt to clear elec angle */
    GPIO_SetFunc(Z_ENCODER, 0);
    /* Config as input */
    GPIO_SetDir(Z_ENCODER, 0);


    /* Init PWM Module */
    PWM_ModuleEnable(PWM1);
    /* Set max encoder cnt */
    PWM_SetModValue(PWM1,PULSE_PER_REVOLUTION - 1);
    /* UP CNT */
    //EncoderConfig.qeiDirection = 1;
    encoderConfig.qeiEnable = 1;
    /* AB phase encode,not cnt and dir encode */
    encoderConfig.qeiMode = 0;
    encoderConfig.qeiPhaFilterEn = 1;
#if (ENCODER_POLARITY > 0)
    encoderConfig.qeiPhaPolarity = 1;
#else
    encoderConfig.qeiPhaPolarity = 0;
#endif
    encoderConfig.qeiPhbFilterEn = 1;
    encoderConfig.qeiPhbPolarity = 0;

    PWM_QEIInit(PWM1, &encoderConfig);
    PWM_SetCallback(PWM1, FOC_Encoder_Callback);
    PWM_EnableOverflowInt(PWM1);

//    NVIC_EnableIRQ(PWM1_IRQn);

    PWM_ClockSet(PWM1, PWM_CLOCK_SYSTEMCLOCK, 0);

    /* Set interrupt priority */
    NVIC_SetPriority(PWM1_IRQn, 2);
}

/*!
* @brief PWM Initialization for debugging, PWM module can be used for debugging output.
*
* @param[in] PWMx: PWM Module
* @return none
*/
void MC_DEBUG_PWM_Initialize(PWM_Type *PWMx)
{

    /*
    * GPIO Configuration for debug PWM
    * PB12    --> PWM0_CH2
    * PB11    --> PWM0_CH3
    */
    /* PWM0_CH0 */
    GPIO_SetFunc(DEBUG_PWM1, 1);
    /* PWM0_CH1 */
    GPIO_SetFunc(DEBUG_PWM2, 1);

    /* Config DEBUG-PWM Interface */
    PWMx->FUNCSEL |= PWM_FUNCSEL_WPDIS_Msk;
    PWM_PWMInit(PWMx, PWM_PWMMODE_EDGEALLIGNED, PWM_PWM_HIGHTRUEPULSE);
    PWM_SetModValue(PWMx, 4096);
    PWM_ClockSet(PWMx, 1, 0);
}

/*!
* @brief SPI Initialization, SPI can be used when external DAC is uesed for debugging.
*
* @param[in] none
* @return none
*/
void MC_DEBUG_SPI_Initialize(void)
{
    SPI_ConfigType spiConfig = {0};
    /*
    * GPIO Configuration for SPI
    * PB0     --> MISO
    * PB1     --> CS
    * PB15    --> SCK
    * PB14    --> MOSI
    */

     /* Set SPI1 GPIO Pinmux */
    GPIO_SetFunc(SPI_NSS, 1);
    GPIO_SetFunc(SPI_SCK, 1);
    GPIO_SetFunc(SPI_MISO, 1);
    GPIO_SetFunc(SPI_MOSI, 1);

    spiConfig.setting.master = ENABLE;
    spiConfig.setting.intEn = DISABLE;
    spiConfig.setting.spiEn = ENABLE;
    spiConfig.setting.dmaRxEn = DISABLE;
    spiConfig.setting.dmaTxEn = ENABLE;
    /* prescale is low + high + 2 */
    spiConfig.sckHigh = 2;
    /* baudrate = 48M / prescale  */
    spiConfig.sckLow = 2;
    /* Frame size is [3:15], stands for 4-16 bits */
    spiConfig.setting.frmSize = 15;
    spiConfig.setting.cpha = 0;
    spiConfig.setting.cpol = 1;
    spiConfig.setting.txMsbFirst = ENABLE;
    spiConfig.setting.rxMsbFirst = ENABLE;
    spiConfig.setting.csOutputEn = ENABLE;
    spiConfig.setting.continuousCSEn = DISABLE;

    SPI_Initialize(SPI1, &spiConfig);
}

/*!
* @brief Motor control debug function initialization, SPI and PWM can be used for debugging output.
*
* @param[in] none
* @return none
*/
void MC_DEBUG_Initialize(void)
{
    if (DAC_PWM == ENABLE)
    {
        MC_DEBUG_PWM_Initialize(PWM0);
    }
    if (DAC_SPI == ENABLE)
    {
        MC_DEBUG_SPI_Initialize();
    }
}

/*!
* @brief FOC PWM Module Initialize.
*
* @param[in] none
* @return none
*/
void FOC_PWM_Initialize(void)
{
    /* Init PWM Channel */
    /*
    * GPIO Configuration for PWM2
    */
    /* PWM2_CH0 */
    GPIO_SetFunc(AH_PWM, 1);
    /* PWM2_CH1 */
    GPIO_SetFunc(AL_PWM, 1);
    /* PWM2_CH2 */
    GPIO_SetFunc(BH_PWM, 2);
    /* PWM2_CH3 */
    GPIO_SetFunc(BL_PWM, 2);
    /* PWM2_CH4 */
    GPIO_SetFunc(CH_PWM, 2);
    /* PWM2_CH5 */
    GPIO_SetFunc(CL_PWM, 2);
    /* Enable PWM Fault1 GPIO Function */
    GPIO_SetFunc(TRAP_PWM_FAULT, 2);

    /* set clock */
    CKGEN_Enable(CLK_PWM2_TIMER, 1);
    CKGEN_Enable(CLK_PWM2_APB, 1);
    CKGEN_SoftReset(SRST_PWM2_APB, 1);
    CKGEN_SoftReset(SRST_PWM2_TIMER, 1);
    /* set driver initial output level */
#if (defined GATE_DRIVER_HIGH_HIGH)
    PWM_OutputInit_Config(PWM2, 0x00);
#elif (defined GATE_DRIVER_LOW_LOW)
    PWM_OutputInit_Config(PWM2, 0x3F);
#elif (defined GATE_DRIVER_HIGH_LOW)
    PWM_OutputInit_Config(PWM2, 0x2A);
#elif (defined GATE_DRIVER_LOW_HIGH)
    PWM_OutputInit_Config(PWM2, 0x15);
#endif
    /* enable initial output control */
    PWM_EnableInitChlOutput(PWM2);

#if (defined GATE_DRIVER_HIGH_HIGH)
    PWM_PWMInit(PWM2, PWM_PWMMODE_COMBINE, PWM_PWM_HIGHTRUEPULSE);
#elif (defined GATE_DRIVER_HIGH_LOW)
    PWM_PWMInit(PWM2, PWM_PWMMODE_COMBINE, PWM_PWM_HIGHTRUEPULSE);
    PWM_ComplementaryModeConfig(PWM2, DISABLE);
#elif (defined GATE_DRIVER_LOW_HIGH)
    PWM_PWMInit(PWM2, PWM_PWMMODE_COMBINE, PWM_PWM_LOWTRUEPULSE);
    PWM_ComplementaryModeConfig(PWM2, DISABLE);
#elif (defined GATE_DRIVER_LOW_LOW)
    #if (PHASE_UVW_POLARITY == ACTIVE_HIGH)
    PWM_PWMInit(PWM2, PWM_PWMMODE_COMBINE, PWM_PWM_HIGHTRUEPULSE);
    #else
    PWM_PWMInit(PWM2, PWM_PWMMODE_COMBINE, PWM_PWM_LOWTRUEPULSE);
    #endif
#endif

    PWM_SetModValue(PWM2, (PWM_PERIOD_VALUE - 1));
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL0, 0);
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL2, 0);
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL4, 0);
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL1, 0);
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL3, 0);
    PWM_SetChannelValue(PWM2, PWM_CHANNEL_CHANNEL5, 0);
#if (defined GATE_DRIVER_HIGH_HIGH)
    /* Active High, up edge input dead time */
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL0, 0);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL2, 0);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL4, 0);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL1, 0);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL3, 0);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL5, 0);
#elif (defined GATE_DRIVER_HIGH_LOW)
    /* Active High, up edge input dead time */
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL0, 0);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL2, 0);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL4, 0);
    /* Active Low, down edge input dead time */
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL1, 1);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL3, 1);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL5, 1);
#elif (defined GATE_DRIVER_LOW_HIGH)
    /* Active Low, down edge input dead time */
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL0, 1);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL2, 1);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL4, 1);
    /* Active High, up edge input dead time */
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL1, 0);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL3, 0);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL5, 0);
#elif (defined GATE_DRIVER_LOW_LOW)
    /* Active Low, down edge input dead time */
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL0, 1);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL2, 1);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL4, 1);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL1, 1);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL3, 1);
    PWM_OutputPolaritySet(PWM2, PWM_CHANNEL_CHANNEL5, 1);
#endif
    /* disable PWM output */
    PWM_Output(PWM_OUTPUT_DISABLE);
    /* 1us Deadtime */
    PWM_PWMDeadtimeSet(PWM2, 2, PWM_DEAD_TIME);
    PWM_SetFaultPirority(PWM2, PWM_EXTERNAL_FAULT1, 1);
    PWM_FaultEnable(PWM2, PWM_EXTERNAL_FAULT1);
    PWM_FaultIntEnable(PWM2);
    PWM_FaultModeSel(PWM2, PWM_FAULT_MANUAL_CLEAR);

    /* set PWM2 Init Trigger Source */
    PWM_SetExternalTrigger(PWM2, PWM_EXTTRIG_INITTRIGEN_Msk);
    PWM_SetCallback(PWM2, FOC_PWM_FaultCallback);
    NVIC_EnableIRQ(PWM2_IRQn);
    /* PWM Start */
    PWM_ClockSet(PWM2, PWM_CLOCK_SYSTEMCLOCK, 0);
    NVIC_SetPriority(PWM2_IRQn, 0);
}

/*!
* @brief FOC_TIM_Initialize(timer Initialize), generates timer interrupt.
*
* @param[in] none
* @return none
*/
void FOC_TIM_Initialize(void)
{
    TIMER_ConfigType timerConfig = {0};
    /* config 500us delay to do pid calculation */
    timerConfig.loadValue = 47999;
    timerConfig.interruptEn = 1;
    timerConfig.linkMode = 0;
    timerConfig.timerEn = 1;

    TIMER_SetCallback(TIMER1, Timer1_Callback);
    TIMER_Init(TIMER1, &timerConfig);
    /* Set interrupt priority */
    NVIC_SetPriority(TIMER1_IRQn, 3);
}

/*!
* @brief Encoder_TIM_Initialize(timer Initialize), generates timer interrupt for M_T speed measure.
*
* @param[in] none
* @return none
*/
void Encoder_TIM_Initialize(void)
{
    TIMER_ConfigType timerConfig = {0};
    timerConfig.loadValue = M_T_PERIOD;
    timerConfig.interruptEn = 1;
    timerConfig.linkMode = 0;
    timerConfig.timerEn = 1;

    TIMER_SetCallback(TIMER2, Timer2_Callback);
    TIMER_Init(TIMER2, &timerConfig);
    /* Set interrupt priority */
    NVIC_SetPriority(TIMER2_IRQn, 1);
}

/*!
* @brief CTU Module Initialize.
*
* @param[in] none
* @return none
*/
void FOC_CTU_Initialize(void)
{
    CTU_ModuleEnable();
    CTU_TriggerAdcInjectGroupByPwm2Init();
    #if (PHASE_UVW_POLARITY == ACTIVE_LOW)
    CTU_SetBusDivide(0);
    CTU_SetPwm2Trig2AdcDelayCnt(0);
    #else
    /* PWM Init Trigger Delay Count Clock prescale, Clock = Bus Clock / 8 */
    CTU_SetBusDivide(5);
    CTU_SetPwm2Trig2AdcDelayCnt(((PWM_PERIOD_VALUE + 1) >> 6));
    #endif
}

/*!
* @brief DMA Module Initialize.
*
* @param[in] none
* @return none
*/
void FOC_DMA_Init()
{
    uint32_t tmpMemStartAddr = (uint32_t)&g_ADCRegularBuffer[0];
    /* Setting memory DMA address */
    uint32_t tmpMemEndAddr = (uint32_t)&g_ADCRegularBuffer[5];
    DMA_ConfigType tmpDMAConfig = {0};

    tmpDMAConfig.memStartAddr = tmpMemStartAddr;
    tmpDMAConfig.memEndAddr = tmpMemEndAddr;
    /* Move ADC DR to memory */
    tmpDMAConfig.periphStartAddr = (uint32_t)(&(ADC->RDR));
    /* Enable DMAx channel */
    tmpDMAConfig.setting.channelEn = 1;
    /* Enable DMA transfer complete interrupt */
    tmpDMAConfig.setting.finishIntEn = 1;
    /* Disable DMA transfer half complete interrupt */
    tmpDMAConfig.setting.halfFinishIntEn = 0;
    tmpDMAConfig.setting.errorIntEn = 1;
    /* 3: Very high priority */
    tmpDMAConfig.setting.channelPriority = 3;
    /* Circular mode enable,If want work one time disable */
    tmpDMAConfig.setting.circular = 1;
    /* 1: Read from peripheral */
    tmpDMAConfig.setting.direction = 0;
    /* Peripheral to memory */
    tmpDMAConfig.setting.MEM2MEM = 0;
    /* 1:16-bit[15:0]; 2:[23:16],[7:0] */
    tmpDMAConfig.setting.memByteMode = 0;
    /* Memory address increment */
    tmpDMAConfig.setting.memIncrement = 1;
    /* Peripheral address fixed */
    tmpDMAConfig.setting.periphIncrement = 0;
    /* 2:32-bit */
    tmpDMAConfig.setting.memSize = 2;
    /* Fixed 1:16-bit */
    tmpDMAConfig.setting.periphSize = 1;
    /* As request */
    tmpDMAConfig.setting.transferNum = 4;

    /* Use DMA1 channel */
    DMA_Initialize(DMA1, &tmpDMAConfig);
    ADC_DMACmd(ADC, ADC_ENABLE);
}

/*!
* @brief Calbiration ADC to get adc channel offset value.
*
* @param[in] sequencechannel : ADC Inject Group sequence num, 0-3
* @return dcoffset: ADC Inject Group channel offset
*/
int16_t MC_ADC_Calbiration(uint8_t sequencechannel)
{
    uint16_t index = 0;
    uint32_t sumOffset = 0;
    int16_t adcOffset = 0;

    ADC_TrigSourceType CalbirationTrigSource = {0, 0};
    ADC_TrigSourceType* calAdcTrigSource;
    calAdcTrigSource = &CalbirationTrigSource;

    calAdcTrigSource->regularTrigSource = ADC_TrigSource_Internal;
    calAdcTrigSource->injectTrigSource = ADC_TrigSource_Internal;
    ADC_TrigSourceInit(ADC, calAdcTrigSource);
    ADC_ClearFlag(ADC, ADC_FLAG_IEOC);
    udelay(10);

    for (index = 0; index < NUM_OF_CONVERSIONS; index ++)
    {
        ADC_SoftwareStartInjectConvCmd(ADC, ADC_ENABLE);
#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
        udelay(10);
        ADC_SoftwareStartInjectConvCmd(ADC, ADC_ENABLE);
#endif
        while (!ADC_GetIntFlag(ADC, ADC_FLAG_IEOC));

        sumOffset += ADC_GetInjectedConversionValue(ADC, sequencechannel);

        ADC_ClearFlag(ADC, ADC_FLAG_IEOC);
    }
    adcOffset = sumOffset / NUM_OF_CONVERSIONS;

    return adcOffset;
}

/*!
* @brief Get phase current sampling offset value.
*
* @param[in] adcSampleReal: pointer to ADC_SAMPLE structure
* @return none
*/
void FOC_ADC_Calbiration(ADC_SAMPLE *adcSampleReal)
{
#if (CUR_SAMPLE_MODE == SINGLE_SHUNT_SAMPLE)
    adcSampleReal->ibusOffset = MC_ADC_Calbiration(ADC_InjectedChannel_1);
#elif ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    adcSampleReal->iaOffset = MC_ADC_Calbiration(ADC_InjectedChannel_1);
    adcSampleReal->ibOffset = MC_ADC_Calbiration(ADC_InjectedChannel_2);
#elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    adcSampleReal->iaOffset = MC_ADC_Calbiration(ADC_InjectedChannel_1);
    adcSampleReal->ibOffset = MC_ADC_Calbiration(ADC_InjectedChannel_2);
    adcSampleReal->icOffset = MC_ADC_Calbiration(ADC_InjectedChannel_3);
#else

#endif
}

/*!
* @brief ADC Initialize, first calibration current Offset, then config adc as external trigger mode.
*
* @param[in] none
* @return none
*/
void FOC_ADC_Initialize(void)
{
//    ADC_DeInit(ADC);
    ADC_InitType tempAdcConfig = {0, 0, 0, 0, 0, 0, 1};
    ADC_InitType* adcConfig;

    ADC_TrigSourceType tempAdcTrigSource = {0, 0};
    ADC_TrigSourceType* adcTrigSource;
    adcTrigSource = &tempAdcTrigSource;
    adcConfig = &tempAdcConfig;

    /*
    * GPIO Configuration for ADC
    * PA7     --> ADC_IN4
    * PA5     --> ADC_IN5
    * PA4     --> ADC_IN6
    * PA3     --> ADC_IN7
    * PA2     --> ADC_IN8
    * PC1     --> ADC_IN9
    * PC0     --> ADC_IN10
    */

    GPIO_SetFunc(IA_ADC, 1);
    GPIO_SetFunc(IB_ADC, 1);
    GPIO_SetFunc(IC_ADC, 1);
    GPIO_SetFunc(VBUS_ADC, 1);
    GPIO_SetFunc(IAVE_ADC, 1);

    ADC_ModuleEnable(ADC);
    /* mode 3 */
    adcConfig->scanMode = 1;
    adcConfig->continousMode = 0;
    adcConfig->disContinousModeOnRegularGroup = 0;
    adcConfig->disContinousModeOnInjectGroup = 0;
    adcConfig->injectAutoMode = 0;

    ADC_SetClockPrescaler(ADC, 4);
    #if ((CUR_SAMPLE_MODE == TWO_SHUNT_SAMPLE) || (CUR_SAMPLE_MODE == ICS_SENSORS_SAMPLE))
    ADC_SetInjectGroupLength(ADC,3);
    ADC_SetRegularGroupLength(ADC, 2);
    ADC_SetInjectGroupSequence(ADC, 1, CURRENT_IA);
    ADC_SetInjectGroupSequence(ADC, 2, CURRENT_IB);
    ADC_SetInjectGroupSequence(ADC, 3, CURRENT_IBUS);

    ADC_SetRegularGroupSequence(ADC, 1, ADC_CHANNEL_AD12);
    ADC_SetRegularGroupSequence(ADC, 2, VOLTAGE_VBUS);
    ADC_ChannelSampleTimeSel(ADC, CURRENT_IA, ADC_SampleTime_6Cycle);
    ADC_ChannelSampleTimeSel(ADC, CURRENT_IB, ADC_SampleTime_6Cycle);
    ADC_ChannelSampleTimeSel(ADC, CURRENT_IBUS, ADC_SampleTime_6Cycle);
    ADC_ChannelSampleTimeSel(ADC, VOLTAGE_VBUS, ADC_SampleTime_6Cycle);
    ADC_ChannelSampleTimeSel(ADC, ADC_CHANNEL_AD12, ADC_SampleTime_6Cycle);

    #elif (CUR_SAMPLE_MODE == THREE_SHUNT_SAMPLE)
    ADC_SetInjectGroupLength(ADC,3);
    ADC_SetRegularGroupLength(ADC, 2);
    ADC_SetInjectGroupSequence(ADC, 1, CURRENT_IA);
    ADC_SetInjectGroupSequence(ADC, 2, CURRENT_IB);
    ADC_SetInjectGroupSequence(ADC, 3, CURRENT_IC);

    ADC_SetRegularGroupSequence(ADC, 1, ADC_CHANNEL_AD12);
    ADC_SetRegularGroupSequence(ADC, 2, VOLTAGE_VBUS);
    ADC_ChannelSampleTimeSel(ADC, CURRENT_IA, ADC_SampleTime_6Cycle);
    ADC_ChannelSampleTimeSel(ADC, CURRENT_IB, ADC_SampleTime_6Cycle);
    ADC_ChannelSampleTimeSel(ADC, CURRENT_IC, ADC_SampleTime_6Cycle);
    ADC_ChannelSampleTimeSel(ADC, VOLTAGE_VBUS, ADC_SampleTime_6Cycle);
    ADC_ChannelSampleTimeSel(ADC, ADC_CHANNEL_AD12, ADC_SampleTime_6Cycle);
    #else
    #endif

    /* ADC works Channel Config */
    ADC_Init(ADC, adcConfig);
    ADC_LP0Disable(ADC);
    ADC_LP1Disable(ADC);
    ADC_CalibrationDisable(ADC);
    ADC_SetCallback(ADC, Motor_Ctrl_CallBack);
    ADC_Cmd(ADC, ADC_ENABLE);

    FOC_ADC_Calbiration(&g_adcSampleReal);

    adcTrigSource->regularTrigSource = ADC_TrigSource_Internal;
    adcTrigSource->injectTrigSource = ADC_TrigSource_External;
    ADC_TrigSourceInit(ADC, adcTrigSource);


    /* Enable Inject Group Interrupt */
    ADC_InjectConvInterruptEnable(ADC);
    /* Start adc convert */
    ADC_SoftwareStartRegularConvCmd(ADC, ADC_ENABLE);
//    NVIC_EnableIRQ(ADC_IRQn);
    NVIC_SetPriority(ADC_IRQn, 1);
}

/*!
* @brief Peripheral Initialize in foc App.
*
* @param[in] none
* @return none
*/
void FOC_Peripheral_Initialize(void)
{
    FOC_GPIO_Initialize();
    FOC_TIM_Initialize();
    FOC_CTU_Initialize();
    FOC_ADC_Initialize();
//    SpeedControlByPWM_init();
    FOC_DMA_Init();
    FOC_PWM_Initialize();
    PWM_Output(PWM_OUTPUT_DISABLE);

#if (defined HALL_SENSORS)
    /* pwdt initialize should before HallSensor_Init */
    FOC_PWDT_Initialize();
#endif
#if (defined SENSOR_ACIM)
    FOC_Encoder_Initialize();
    #if (defined M_T_SPEED_MEASURE)
    Encoder_TIM_Initialize();
    #endif
#elif (defined SENSORLESS_ACIM)
#else
#endif


    MC_DEBUG_Initialize();
#if (defined ENCODER_SENSORS)
    FOC_Encoder_Initialize();
    #if (defined M_T_SPEED_MEASURE)
    Encoder_TIM_Initialize();
    #endif
#elif (defined ENCODER_HALL_SENSORS)
    FOC_PWDT_Initialize();
    FOC_Encoder_Initialize();
    #if (defined M_T_SPEED_MEASURE)
    Encoder_TIM_Initialize();
    #endif
#else
#endif

    NVIC_EnableIRQ(ADC_IRQn);
}

/* =============================================  EOF  ============================================== */
